@@ -19,6 +19,7 @@
 #include <string.h>
 #include <signal.h>
 #include <unistd.h>
+#include <getopt.h>
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/socket.h>
@@ -44,13 +45,14 @@ static volatile int stop = 0;
 
 int quiet = 0;
 char machine_readable = 0;
+
 char nagios_mode = 0;
 
 char last_error[ERROR_BUFFER_SIZE];
 
 void version(void)
 {
-	fprintf(stderr, "HTTPing v" VERSION ", (C) 2003-2009 folkert@vanheusden.com\n");
+	fprintf(stderr, "HTTPing v" VERSION ", (C) 2003-2010 folkert@vanheusden.com\n");
 #ifndef NO_SSL
 	fprintf(stderr, "SSL support included\n");
 #endif
@@ -58,48 +60,91 @@ void version(void)
 
 void usage(void)
 {
-	fprintf(stderr, "\n-g url		url (e.g. -g http://localhost/)\n");
-	fprintf(stderr, "-h hostname	hostname (e.g. localhost)\n");
-	fprintf(stderr, "-p portnr	portnumber (e.g. 80)\n");
-	fprintf(stderr, "-x host:port	hostname+portnumber of proxyserver\n");
-	fprintf(stderr, "-c count	how many times to connect\n");
-	fprintf(stderr, "-i interval	delay between each connect\n");
-	fprintf(stderr, "-t timeout	timeout (default: 30s)\n");
-	fprintf(stderr, "-s		show statuscodes\n");
+	fprintf(stderr, "\n-g url         url (e.g. -g http://localhost/)\n");
+	fprintf(stderr, "-h hostname    hostname (e.g. localhost)\n");
+	fprintf(stderr, "-p portnr      portnumber (e.g. 80)\n");
+	fprintf(stderr, "-x host:port   hostname+portnumber of proxyserver\n");
+	fprintf(stderr, "-c count       how many times to connect\n");
+	fprintf(stderr, "-i interval    delay between each connect\n");
+	fprintf(stderr, "-t timeout     timeout (default: 30s)\n");
+	fprintf(stderr, "-6             use IPv6\n");
+	fprintf(stderr, "-s             show statuscodes\n");
 	fprintf(stderr, "-S             split time in connect-time and processing time\n");
-	fprintf(stderr, "-G		do a GET request instead of HEAD (read the\n");
-	fprintf(stderr, "		contents of the page as well)\n");
-	fprintf(stderr, "-b		show transfer speed in KB/s (use with -G)\n");
-	fprintf(stderr, "-B		like -b but use compression if available\n");
-	fprintf(stderr, "-L x		limit the amount of data transferred (for -b)\n");
-	fprintf(stderr, "		to 'x' (in bytes)\n");
-	fprintf(stderr, "-X		show the number of KB transferred (for -b)\n");
+	fprintf(stderr, "-G             do a GET request instead of HEAD (read the\n");
+	fprintf(stderr, "               contents of the page as well)\n");
+	fprintf(stderr, "-b             show transfer speed in KB/s (use with -G)\n");
+	fprintf(stderr, "-B             like -b but use compression if available\n");
+	fprintf(stderr, "-L x           limit the amount of data transferred (for -b)\n");
+	fprintf(stderr, "               to 'x' (in bytes)\n");
+	fprintf(stderr, "-X             show the number of KB transferred (for -b)\n");
 #ifndef NO_SSL
-	fprintf(stderr, "-l		connect using SSL\n");
-	fprintf(stderr, "-z		show fingerprint (SSL)\n");
+	fprintf(stderr, "-l             connect using SSL\n");
+	fprintf(stderr, "-z             show fingerprint (SSL)\n");
 #endif
-	fprintf(stderr, "-f		flood connect (no delays)\n");
-	fprintf(stderr, "-a		audible ping\n");
-	fprintf(stderr, "-m		give machine parseable output (see\n");
-	fprintf(stderr, "		also -o and -e)\n");
-	fprintf(stderr, "-o rc,rc,...	what http results codes indicate 'ok'\n");
-	fprintf(stderr, "		coma seperated WITHOUT spaces inbetween\n");
-	fprintf(stderr, "		default is 200, use with -e\n");
-	fprintf(stderr, "-e str		string to display when http result code\n");
-	fprintf(stderr, "		doesn't match\n");
-	fprintf(stderr, "-I str		use 'str' for the UserAgent header\n");
-	fprintf(stderr, "-R str		use 'str' for the Referer header\n");
-	fprintf(stderr, "-r		resolve hostname only once (usefull when\n");
-	fprintf(stderr, "		pinging roundrobin DNS: also takes the first\n");
-	fprintf(stderr, "		DNS lookup out of the loop so that the first\n");
-	fprintf(stderr, "		measurement is also correct)\n");
-	fprintf(stderr, "-n warn,crit	Nagios-mode: return 1 when avg. response time\n");
-	fprintf(stderr, "		>= warn, 2 if >= crit, otherwhise return 0\n");
-	fprintf(stderr, "-N x		Nagios mode 2: return 0 when all fine, 'x'\n");
-	fprintf(stderr, "		when anything failes\n");
+	fprintf(stderr, "-f             flood connect (no delays)\n");
+	fprintf(stderr, "-a             audible ping\n");
+	fprintf(stderr, "-m             give machine parseable output (see\n");
+	fprintf(stderr, "               also -o and -e)\n");
+	fprintf(stderr, "-o rc,rc,...   what http results codes indicate 'ok'\n");
+	fprintf(stderr, "               coma seperated WITHOUT spaces inbetween\n");
+	fprintf(stderr, "               default is 200, use with -e\n");
+	fprintf(stderr, "-e str         string to display when http result code\n");
+	fprintf(stderr, "               doesn't match\n");
+	fprintf(stderr, "-I str         use 'str' for the UserAgent header\n");
+	fprintf(stderr, "-R str         use 'str' for the Referer header\n");
+	fprintf(stderr, "-r             resolve hostname only once (usefull when\n");
+	fprintf(stderr, "               pinging roundrobin DNS: also takes the first\n");
+	fprintf(stderr, "               DNS lookup out of the loop so that the first\n");
+	fprintf(stderr, "               measurement is also correct)\n");
+	fprintf(stderr, "-n warn,crit   Nagios-mode: return 1 when avg. response time\n");
+	fprintf(stderr, "               >= warn, 2 if >= crit, otherwhise return 0\n");
+	fprintf(stderr, "-N x           Nagios mode 2: return 0 when all fine, 'x'\n");
+	fprintf(stderr, "               when anything failes\n");
 	fprintf(stderr, "-y ip[:port]   bind to ip-address (and thus interface) [/port]\n");
-	fprintf(stderr, "-q		quiet, only returncode\n");
-	fprintf(stderr, "-V		show the version\n\n");
+	fprintf(stderr, "-q             quiet, only returncode\n");
+	fprintf(stderr, "-A             Activate Basic authentication\n");
+	fprintf(stderr, "-U Username    needed for authentication\n");
+	fprintf(stderr, "-P Password    needed for authentication\n");
+	fprintf(stderr, "-C cookie=value Add a cookie to the request\n");
+	fprintf(stderr, "-V             show the version\n\n");
+	fprintf(stderr, "\n");
+	fprintf(stderr, "--url			-g\n");
+	fprintf(stderr, "--hostname		-h\n");
+	fprintf(stderr, "--port			-p\n");
+	fprintf(stderr, "--host-port		-x\n");
+	fprintf(stderr, "--count		-c\n");
+	fprintf(stderr, "--interval		-i\n");
+	fprintf(stderr, "--timeout		-t\n");
+	fprintf(stderr, "--ipv6		-	6\n");
+	fprintf(stderr, "--show-statusodes	-s\n");
+	fprintf(stderr, "--split-time		-S\n");
+	fprintf(stderr, "--get-request		-G\n");
+	fprintf(stderr, "--show-transfer-speed	-b\n");
+	fprintf(stderr, "--show-xfer-speed-compressed		-B\n");
+	fprintf(stderr, "--data-limit		-L\n");
+	fprintf(stderr, "--show-kb		-X\n");
+#ifndef NO_SSL
+	fprintf(stderr, "--use-ssl		-l\n");
+	fprintf(stderr, "--show-fingerprint	-z\n");
+#endif
+	fprintf(stderr, "--flood		-f\n");
+	fprintf(stderr, "--audible-ping		-a\n");
+	fprintf(stderr, "--parseable-output	-m\n");
+	fprintf(stderr, "--ok-result-codes	-o\n");
+	fprintf(stderr, "--result-string	-e\n");
+	fprintf(stderr, "--user-agent		-I\n");
+	fprintf(stderr, "--referer		-S\n");
+	fprintf(stderr, "--resolve-once		-r\n");
+	fprintf(stderr, "--nagios-mode-1	-n\n");
+	fprintf(stderr, "--nagios-mode-2	-n\n");
+	fprintf(stderr, "--bind-to		-y\n");
+	fprintf(stderr, "--quiet		-q\n");
+	fprintf(stderr, "--basic-auth		-A\n");
+	fprintf(stderr, "--username		-U\n");
+	fprintf(stderr, "--password		-P\n");
+	fprintf(stderr, "--cookie		-C\n");
+	fprintf(stderr, "--version		-V\n");
+	fprintf(stderr, "--help			-h\n");
 }
 
 void emit_error()
@@ -118,6 +163,50 @@ void handler(int sig)
 	stop = 1;
 }
 
+/* Base64 encoding start */  
+const char *alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+
+void encode_tryptique(char source[3], char result[4])
+/* Encode 3 char in B64, result give 4 Char */
+ {
+    int tryptique, i;
+    tryptique = source[0];
+    tryptique *= 256;
+    tryptique += source[1];
+    tryptique *= 256;
+    tryptique += source[2];
+    for (i=0; i<4; i++)
+    {
+ 	result[3-i] = alphabet[tryptique%64];
+	tryptique /= 64;
+    }
+} 
+
+
+int enc_b64(char *source, size_t source_lenght, char *target)
+{
+	/* Divide string /3 and encode trio */
+	while (source_lenght >= 3) {
+		encode_tryptique(source, target);
+		source_lenght -= 3;
+		source += 3;
+		target += 4;
+	}
+	/* Add padding to the rest */
+	if (source_lenght > 0) {
+		char pad[3];
+	 	memset(pad, 0, sizeof(pad));
+		memcpy(pad, source, source_lenght);
+		encode_tryptique(pad, target);
+		target[3] = '=';
+		if (source_lenght == 1) target[2] = '=';
+		target += 4;
+	}
+	target[0] = 0;
+	return 1;
+} 
+/* Base64 encoding END */  
+
 int main(int argc, char *argv[])
 {
 	char *hostname = NULL;
@@ -141,9 +230,14 @@ int main(int argc, char *argv[])
 	char *useragent = NULL;
 	char *referer = NULL;
 	char *host;
+	char *pwd = NULL;
+	char *usr = NULL;
+	char *cookie = NULL;
 	int port;
-	struct sockaddr_in addr;
+	struct sockaddr_in6 addr;
+	struct addrinfo ai;
 	char resolve_once = 0;
+	char auth_mode = 0;
 	char have_resolved = 0;
 	double nagios_warn=0.0, nagios_crit=0.0;
 	int nagios_exit_code = 2;
@@ -159,9 +253,51 @@ int main(int argc, char *argv[])
 	int fd = -1;
 	SSL *ssl_h = NULL;
 	BIO *s_bio;
-	struct sockaddr_in bind_to;
+	struct sockaddr_in6 bind_to;
 	char bind_to_valid = 0;
-	char split = 0;
+	char split = 0, use_ipv6 = 0;
+
+	static struct option long_options[] =
+	{
+		{"url",		1, NULL, 'g' },
+		{"hostname",	1, NULL, 'h' },
+		{"port",	1, NULL, 'p' },
+		{"host-port",	1, NULL, 'x' },
+		{"count",	1, NULL, 'c' },
+		{"interval",	1, NULL, 'i' },
+		{"timeout",	1, NULL, 't' },
+		{"ipv6",	0, NULL, '6' },
+		{"show-statusodes",	0, NULL, 's' },
+		{"split-time",	0, NULL, 'S' },
+		{"get-request",	0, NULL, 'G' },
+		{"show-transfer-speed",	0, NULL, 'b' },
+		{"show-xfer-speed-compressed",	0, NULL, 'B' },
+		{"data-limit",	1, NULL, 'L' },
+		{"show-kb",	0, NULL, 'X' },
+#ifndef NO_SSL
+		{"use-ssl",	0, NULL, 'l' },
+		{"show-fingerprint",	0, NULL, 'z' },
+#endif
+		{"flood",	0, NULL, 'f' },
+		{"audible-ping",	0, NULL, 'a' },
+		{"parseable-output",	0, NULL, 'm' },
+		{"ok-result-codes",	1, NULL, 'o' },
+		{"result-string",	1, NULL, 'e' },
+		{"user-agent",	1, NULL, 'I' },
+		{"referer",	1, NULL, 'S' },
+		{"resolve-once",0, NULL, 'r' },
+		{"nagios-mode-1",	1, NULL, 'n' },
+		{"nagios-mode-2",	1, NULL, 'n' },
+		{"bind-to",	1, NULL, 'y' },
+		{"quiet",	0, NULL, 'q' },
+		{"basic-auth",	0, NULL, 'A' },
+		{"username",	1, NULL, 'U' },
+		{"password",	1, NULL, 'P' },
+		{"cookie",	1, NULL, 'C' },
+		{"version",	0, NULL, 'V' },
+		{"help",	0, NULL, 'h' },
+		{NULL,		0, NULL, 0   }
+	};
 
 	signal(SIGPIPE, SIG_IGN);
 
@@ -170,10 +306,14 @@ int main(int argc, char *argv[])
 
 	buffer = (char *)mymalloc(page_size, "receive buffer");
 
-	while((c = getopt(argc, argv, "Sy:XL:bBg:h:p:c:i:Gx:t:o:e:falqsmV?I:R:rn:N:z")) != -1)
+	while((c = getopt_long(argc, argv, "6Sy:XL:bBg:h:p:c:i:Gx:t:o:e:falqsmV?I:R:rn:N:z:AP:U:C:", long_options, NULL)) != -1)
 	{
 		switch(c)
 		{
+			case '6':
+				use_ipv6 = 1;
+				break;
+
 			case 'S':
 				split = 1;
 				break;
@@ -188,11 +328,11 @@ int main(int argc, char *argv[])
 
 					if (dummy)
 					{
-						bind_to.sin_port = htons(atoi(dummy + 1));
+						bind_to.sin6_port = htons(atoi(dummy + 1));
 						*dummy = 0x00;
 					}
 
-					if (inet_aton(optarg, &bind_to.sin_addr) == 0)
+					if (inet_pton(AF_INET6, optarg, &(bind_to.sin6_addr)) != 1)
 					{
 						error_exit("cannot convert ip address '%s' (for -y)\n", optarg);
 					}
@@ -317,7 +457,18 @@ int main(int argc, char *argv[])
 				nagios_mode = 2;
 				nagios_exit_code = atoi(optarg);
 				break;
-
+			case 'A': 
+				auth_mode = 1; 
+				break;
+			case 'P':
+				pwd = optarg;
+				break;
+			case 'U':
+				usr = optarg;
+				break;
+			case 'C':
+				cookie = optarg;
+				break;
 			case '?':
 			default:
 				version();
@@ -331,6 +482,9 @@ int main(int argc, char *argv[])
 
 	last_error[0] = 0x00;
 
+
+       
+
 	if (!get_instead_of_head && show_Bps)
 		error_exit("-b/-B can only be used when also using -G\n");
 
@@ -364,11 +518,14 @@ int main(int argc, char *argv[])
 		if (slash)
 			*slash = 0x00;
 
-		colon = strchr(getcopy, ':');
-		if (colon)
+		if (!use_ipv6)
 		{
-			*colon = 0x00;
-			portnr = atoi(colon + 1);
+			colon = strchr(getcopy, ':');
+			if (colon)
+			{
+				*colon = 0x00;
+				portnr = atoi(colon + 1);
+			}
 		}
 
 		hostname = getcopy;
@@ -455,6 +612,23 @@ int main(int argc, char *argv[])
 		sprintf(&request[strlen(request)], "Referer: %s\r\n", referer);
 	if (ask_compression)
 		sprintf(&request[strlen(request)], "Accept-Encoding: gzip,deflate\r\n");
+
+	/* Basic Authentification */
+	if (auth_mode) { 
+		char auth_string[255];
+		char b64_auth_string[255];
+		if ( usr == NULL )
+			error_exit("Basic Authnetication (-A) can only be used with a username and/or password (-U -P) ");
+		sprintf(auth_string,"%s:%s",usr,pwd); 
+		enc_b64(auth_string, strlen(auth_string), b64_auth_string);
+		sprintf(&request[strlen(request)], "Authorization: Basic %s\r\n", b64_auth_string);
+	}
+
+	/* Cookie Insertion */
+	if (cookie) { 
+		sprintf(&request[strlen(request)], "Cookie: %s;\r\n", cookie);
+	}
+
 	strcat(request, "\r\n");
 	req_len = strlen(request);
 
@@ -473,7 +647,7 @@ int main(int argc, char *argv[])
 	{
 		memset(&addr, 0x00, sizeof(addr));
 
-		if (resolve_host(host, &addr) == -1)
+		if (resolve_host(host, &addr, &ai, use_ipv6, port) == -1)
 		{
 			err++;
 			emit_error();
@@ -504,7 +678,7 @@ int main(int argc, char *argv[])
 			{
 				memset(&addr, 0x00, sizeof(addr));
 
-				if (resolve_host(host, &addr) == -1)
+				if (resolve_host(host, &addr, &ai, use_ipv6, port) == -1)
 				{
 					err++;
 					emit_error();
@@ -514,7 +688,7 @@ int main(int argc, char *argv[])
 				have_resolved = 1;
 			}
 
-			fd = connect_to(bind_to_valid?(struct sockaddr *)&bind_to:NULL, &addr, port, timeout);
+			fd = connect_to(bind_to_valid?(struct sockaddr *)&bind_to:NULL, &ai, timeout);
 
 			if (fd == -3)	/* ^C pressed */
 				break;
@@ -824,13 +998,13 @@ error_exit:
 			return 1;
 		}
 
-		printf("OK - average httping-time is %.1f (%s)\n", avg_httping_time, last_error);
+		printf("OK - average httping-time is %.1f (%s)|ping=%f\n", avg_httping_time, last_error, avg_httping_time);
 	}
 	else if (nagios_mode == 2)
 	{
 		if (ok && last_error[0] == 0x00)
 		{
-			printf("OK - all fine, avg httping time is %.1f\n", avg_httping_time);
+			printf("OK - all fine, avg httping time is %.1f|ping=%f\n", avg_httping_time, avg_httping_time);
 			return 0;
 		}
 
@@ -843,3 +1017,6 @@ error_exit:
 	else
 		return 127;
 }
+
+
+
