@@ -13,5 +13,5 @@
   files in the program, then also delete it here.
 */
 
-int connect_to(struct sockaddr *bind_to, struct sockaddr_in *addr, int portnr, int timeout);
+int connect_to(struct sockaddr *bind_to, struct addrinfo *ai, int timeout);
 int set_tcp_low_latency(int sock);
